#include <limits>
#include <set>
#include <string>
#include <vector>




namespace soma
  {



  namespace solver
    {



    namespace dlx_algorithm
      {



      template
        <
        typename DataT
        >
      inline void CDLXSomaSolver::generateSparseMatrix
                                                     (
                                                     const DataT& data
                                                     )
        {

        typedef typename DataT::size_type size_type;
        const size_type nbRows=data.size(),
                        nbCols=data[0].size();

        if(

           (nbRows==0)||

           (nbCols<34)||

           !(

             ((data[0][0])&&(!data[0][1])&&(!data[0][2])&&(!data[0][3])&&(!data[0][4])&&(!data[0][5])&&(!data[0][6]))||

             ((!data[0][0])&&(data[0][1])&&(!data[0][2])&&(!data[0][3])&&(!data[0][4])&&(!data[0][5])&&(!data[0][6]))||

             ((!data[0][0])&&(!data[0][1])&&(data[0][2])&&(!data[0][3])&&(!data[0][4])&&(!data[0][5])&&(!data[0][6]))||

             ((!data[0][0])&&(!data[0][1])&&(!data[0][2])&&(data[0][3])&&(!data[0][4])&&(!data[0][5])&&(!data[0][6]))||

             ((!data[0][0])&&(!data[0][1])&&(!data[0][2])&&(!data[0][3])&&(data[0][4])&&(!data[0][5])&&(!data[0][6]))||

             ((!data[0][0])&&(!data[0][1])&&(!data[0][2])&&(!data[0][3])&&(!data[0][4])&&(data[0][5])&&(!data[0][6]))||

             ((!data[0][0])&&(!data[0][1])&&(!data[0][2])&&(!data[0][3])&&(!data[0][4])&&(!data[0][5])&&(data[0][6]))

            )

          )
          throw
              (
              std::invalid_argument
                                  (
                                  "Argument data is not a valid SOMA exact cover problem matrix!"
                                  )
              );

        unsigned int nbOnes=nbRows;

        for(size_type j=7;j<nbCols;++j)
          if(data[0][j])
            ++nbOnes;

        for(size_type i=1;i<nbRows;++i)
          {
          if(

             (data[i].size()!=nbCols)||

             !(

               ((data[i][0])&&(!data[i][1])&&(!data[i][2])&&(!data[i][3])&&(!data[i][4])&&(!data[i][5])&&(!data[i][6]))||

               ((!data[i][0])&&(data[i][1])&&(!data[i][2])&&(!data[i][3])&&(!data[i][4])&&(!data[i][5])&&(!data[i][6]))||

               ((!data[i][0])&&(!data[i][1])&&(data[i][2])&&(!data[i][3])&&(!data[i][4])&&(!data[i][5])&&(!data[i][6]))||

               ((!data[i][0])&&(!data[i][1])&&(!data[i][2])&&(data[i][3])&&(!data[i][4])&&(!data[i][5])&&(!data[i][6]))||

               ((!data[i][0])&&(!data[i][1])&&(!data[i][2])&&(!data[i][3])&&(data[i][4])&&(!data[i][5])&&(!data[i][6]))||

               ((!data[i][0])&&(!data[i][1])&&(!data[i][2])&&(!data[i][3])&&(!data[i][4])&&(data[i][5])&&(!data[i][6]))||

               ((!data[i][0])&&(!data[i][1])&&(!data[i][2])&&(!data[i][3])&&(!data[i][4])&&(!data[i][5])&&(data[i][6]))

              )

            )
            throw
                (
                std::invalid_argument
                                    (
                                    "Argument data is not a valid SOMA exact cover problem matrix!"
                                    )
                );
          for(size_type j=7;j<nbCols;++j)
            if(data[i][j])
              ++nbOnes;
          }


        unsigned int c=1,
                     r=0;
        header_t* columnPtr=0;
        base_node_t* rightPtr=0;
        bool oneInColumn=false;


        m_Headers.resize
                       (
                       nbCols+1
                       );
        m_MatrixElements.resize
                              (
                              nbOnes
                              );
        m_Rows.resize
                    (
                    nbRows
                    );

        rightPtr=m_H=&m_Headers[0];


        for(size_type j=0;j<nbCols;++j)
          {

          for(size_type i=0;i<nbRows;++i)
            {

            if(data[i][j])
              {

              if(!oneInColumn)
                {

                (rightPtr->right=&m_Headers[c++])->left=rightPtr;
                (columnPtr=(header_t*)(rightPtr=rightPtr->right))->size=0;
                columnPtr->column=(j<7)?j:j-7;
                oneInColumn=true;

                }

              (rightPtr->down=&m_MatrixElements[r++])->up=rightPtr;
              rightPtr=rightPtr->down;
              m_Rows[((matrix_element_t*)rightPtr)->row=i].push_back
                                                                   (
                                                                   (matrix_element_t*)rightPtr
                                                                   );
              ((matrix_element_t*)rightPtr)->header=columnPtr;
              ++columnPtr->size;

              }

            }

          if(oneInColumn)
            {
            rightPtr->down=columnPtr;
            columnPtr->up=rightPtr;
            rightPtr=columnPtr;
            oneInColumn=false;
            }

          }


        rightPtr->right=m_H;
        m_H->left=rightPtr;


        for(size_type i=0;i<nbRows;++i)
          {
          const size_type nbOnesInRowI=m_Rows[i].size(),
                          nbOnesInRowIMinus1=nbOnesInRowI-1;

          m_Rows[i][0]->left=m_Rows[i][nbOnesInRowIMinus1];
          m_Rows[i][nbOnesInRowIMinus1]->right=m_Rows[i][0];

          for(size_type j=1;j<nbOnesInRowI;++j)
            {
            m_Rows[i][j]->left=m_Rows[i][j-1];
            m_Rows[i][j-1]->right=m_Rows[i][j];
            }

          }


        m_H->name='H';
        ((header_t*)(m_H->right))->name='A';
        ((header_t*)(m_H->right->right))->name='B';
        ((header_t*)(m_H->right->right->right))->name='L';
        ((header_t*)(m_H->right->right->right->right))->name='P';
        ((header_t*)(m_H->right->right->right->right->right))->name='T';
        ((header_t*)(m_H->right->right->right->right->right->right))->name='V';
        ((header_t*)(m_H->right->right->right->right->right->right->right))->name='Z';

        }



      }



    }



  }
