#ifndef ALL_SOMA_PIECES_POSITIONS_HPP
#define ALL_SOMA_PIECES_POSITIONS_HPP



#include "AllPolyformPiecesPositions.hpp"



namespace soma
  {



  namespace solver
    {



    namespace soma_structure
      {


/*
  La classe CAllSomaPiecesPositions drive de CAllPolyformPiecesPositions. Elle permet de calculer toutes
  les positions possibles des 7 pices A, B, L, P, T, V et Z du jeu SOMA dans une structure  rsoudre.
*/
      class CAllSomaPiecesPositions : public CAllPolyformPiecesPositions<7,CAllSomaPiecesPositions>
        {

        friend class CAllPolyformPiecesPositions<7,CAllSomaPiecesPositions>;


        private :

/*
  Fonction membre prive statique addNewPosToVList. Voir le fichier d'implmentation AllSomaPiecesPositions.cpp pour
  avoir une description plus complte de la fonction addNewPosToVList.

  Arguments 1 et 2 :
    - type : integer_t (ici alias de unsigned short).
    - noms : N2 et N3.
    - N2 et N3 sont le nombre de cubes sur l'axe des Y et des Z, respectivement d'une bote englobante d'une structure S.
      Ils servent pour la formule de conversion d'indices de tableau  3 dimensions vers tableau  1 dimension.

  Argument 3 :
    - type : integer_t (ici alias de unsigned short).
    - nom : N1xN2xN3.
    - N1xN2xN3 est le nombre total de cubes dans la bote englobante  de S.

  Argument 4 :integer_t (ici alias de unsigned short).
    - type : rfrence sur MatrixT (ici alias de std::vector<std::vector<bool> >).
    - nom : somaMatrix.
    - somaMatrix est la matrice boolenne dont les lignes forment l'ensemble de toutes les positions possibles
      de toutes les pices du jeu SOMA dans S.

  Arguments 5  13 :
    - type : integer_t (ici alias de unsigned short).
    - noms : i1, j1, k1, i2, j2, k2, i3, j3 et k3.
    - Les 3 3-upls (i1,j1,k1), (i2, j2 ,k2) et (i3,j3,k3) correspondent aux positions de 3 cubes (diffrents
      sinon computeAllPiecesPositions n'appellerait pas addNewPosToVList...) dans la bote englobante de S.

  Retour : void.
*/
          template
            <
            typename IntegerT1,
            typename IntegerT2,
            typename MatrixT
            >
          inline static void addNewPosToVList
                                            (
                                            const IntegerT1 N2,
                                            const IntegerT1 N3,
                                            const IntegerT2 N1xN2xN3,
                                            MatrixT& somaMatrix,
                                            const IntegerT1 i1,
                                            const IntegerT1 j1,
                                            const IntegerT1 k1,
                                            const IntegerT1 i2,
                                            const IntegerT1 j2,
                                            const IntegerT1 k2,
                                            const IntegerT1 i3,
                                            const IntegerT1 j3,
                                            const IntegerT1 k3
                                            );



/*
  Fonction membre prive statique addNewPosToPieceList. Voir le fichier d'implmentation AllSomaPiecesPositions.cpp pour
  avoir une description plus complte de la fonction addNewPosToPieceList.

  Argument 1 et 2 :
    - type : integer_t (ici alias de unsigned short).
    - noms : N2 et N3.
    - N2 et N3 sont le nombre de cubes sur l'axe des Y et des Z, respectivement d'une bote englobante d'une structure S.
      Ils servent pour la formule de conversion d'indices de tableau  3 dimensions vers tableau  1 dimension.

  Argument 3 :
    - type : integer_t (ici alias de unsigned short).
    - nom : N1xN2xN3.
    - N1xN2xN3 est le nombre total de cubes dans la bote englobante  de S.

  Argument 4 :integer_t (ici alias de unsigned short).
    - type : rfrence sur MatrixT (ici alias de std::vector<std::vector<bool> >).
    - nom : somaMatrix.
    - somaMatrix est la matrice boolenne dont les lignes forment l'ensemble de toutes les positions possibles
      de toutes les pices du jeu SOMA dans S.

  Argument 5 :
    - type : integer_t (ici alias de unsigned short).
    - nom : pieceIndex.
    - pieceIndex est l'indice de la pice pour laquelle computeAllSomaPositions a calcul une position.
      pieceIndex vaut 0 pour A, 1 pour B, 2 pour L, 3 pour P, 4 pour T et 6 pour Z.

  Arguments 6  17 :
    - type : integer_t (ici alias de unsigned short).
    - noms : i1, j1, k1, i2, j2, k2, i3, j3, k3, i4, j4 et k4.
    - Les 4 3-upls (i1,j1,k1), (i2, j2 ,k2), (i3,j3,k3) et (i4,j4,k4) correspondent aux positions de
      4 cubes diffrents dans la bote englobante de S.

  Retour : void.
*/
          template
            <
            typename IntegerT1,
            typename IntegerT2,
            typename MatrixT
            >
          inline static void addNewPosToPieceList
                                                (
                                                const IntegerT1 N2,
                                                const IntegerT1 N3,
                                                const IntegerT2 N1xN2xN3,
                                                MatrixT& somaMatrix,
                                                const IntegerT1 pieceIndex,
                                                const IntegerT1 i1,
                                                const IntegerT1 j1,
                                                const IntegerT1 k1,
                                                const IntegerT1 i2,
                                                const IntegerT1 j2,
                                                const IntegerT1 k2,
                                                const IntegerT1 i3,
                                                const IntegerT1 j3,
                                                const IntegerT1 k3,
                                                const IntegerT1 i4,
                                                const IntegerT1 j4,
                                                const IntegerT1 k4
                                                );



/*
  Fonction membre prive computeAllPiecesPositions. Voir le fichier d'implmentation AllSomaPiecesPositions.cpp pour
  avoir une description plus complte de la fonction computeAllPiecesPositions.

  Argument : Aucun.

  Retour : void
*/
          inline void computeAllPiecesPositions();


/*
  Unique constructeur publique de la classe CAllSomaPiecesPositions. Il appelle celui de la classe mre CAllPolyformPiecesPositions.

  Argument :
    - type : const bit_array_t& boundingBox. En gnral, bit_array_t est un alias de soma::utils::general::CBitArray3D.
    - nom : boundingBox.
    - boundingBox est la bote englobante de la structure  rsoudre.
*/
        public :

          CAllSomaPiecesPositions
                                (
                                const bit_array_t& boundingBox
                                ) : CAllPolyformPiecesPositions<7,CAllSomaPiecesPositions>
                                                                                         (
                                                                                         boundingBox
                                                                                         ) {}

        };



      }



    }



  }



#endif  //  ALL_SOMA_PIECES_POSITIONS_HPP
