#include <fstream>
#include <string>
#include <sstream>
#include <stdexcept>



#include "SomaLoader.hpp"
#include "../../utils/general/IsOfType.hpp"



namespace soma
  {



  namespace solver
    {



    namespace soma_structure
      {



/*
  Unique constructeur public de la classe CSomaLoader. Reoit le nom du fichier  charger. Ce fichier doit respecter scrupuleusement le format impos par le rglement du jeu.
  Il est uniquement permis de sauter des lignes entre chaque ligne de coordonnes (mais pas aprs la dernire ligne) ou de mettre des espaces aprs chaque coordonne au sein
  d'une ligne (mais pas aprs la dernire coordonne). Il n'est pas obligatoire qu'il y ait exactement 27 cubes de coordonnes diffrentes, mais il doit y avoir exactement 27
  cubes de coordonnes au format valide (x, y, et z spars par des virgules et compris entre 0 et 9 inclus). Si ces conditions ne sont pas respectes, alors le constructeur
  lance une exception de type std::invalid_argument.

  Argument :
    - type : const char*.
    - nom : fileName.
    - fileName contient le nom du fichier  charger.
*/
      CSomaLoader::CSomaLoader
                             (
                             const char* fileName
                             )
        {

// On dclare un flux de fichier fileStream de type std::ifstream pour charger les coordonnes d'une structure  rsoudre.
        std::ifstream fileStream
                               (
                               fileName
                               );
//  fileStream.fail() permet de tester si le fichier est accessible en lecture.
//  Si le test choue alors le fichier est inexistant ou inaccessible  la lecture.
//  Dans ce cas l, on lance alors une exception de type std::runtime_error car l'echec
//  d'ouverture ou de lecture peut ne pas forcment tre d  une mauvaise manipulation de l'utilisateur.
        if(fileStream.fail())
          throw
              (
              std::runtime_error
                               (
                               "File does not exist or cannot be read!"
                               )
              );

        std::string line, // Contient une ligne du fichier.
                    word; // Contient une coordonne (x,y ou z) d'un cube.
        unsigned char nbReadLines=0, // Nombre de lignes lues dans le fichier.
                      nbReadWords=0, // Nombre de coordonnes lues par ligne.
                      cubeletsPositions[27][3], // Tableau contenant les coordonnes des cubes "brutes" (non centres en (0,0,0).
                      minAndMax[][2]= // Tableau contenant les valeurs minimales et maximales des coordonnes lues dans le fichier.
                                    {
                                     {
                                     10, // x minimal lu.
                                     0 // x maximal lu.
                                     },
                                     {
                                     10, // y minimal lu.
                                     0 // y maximal lu.
                                     },
                                     {
                                     10,  // z minimal lu.
                                     0 // z maximal lu.
                                     }
                                    };
        int tmpVal;
        bool remainLinesToRead=true; // boolen indiquant s'il reste des lignes  lire dans le fichier.

//  On lit dans le fichier jusqu' ce qu'il n'ait plus de lignes dans le fichier ou jusqu' ce que l'on ait lu 27 vraies coordonnes valide de cubes.
        while((remainLinesToRead)&&(nbReadLines<27))
          {
//  On vrifie s'il y a encore des lignes  lire dans le fichier. Si c'est le cas, on assigne  line la premire ligne restante.
          remainLinesToRead=std::getline(fileStream,line);
//  Il est permis de sauter des lignes entre chaque coordonne lue. Si on rencontre une ligne vide, on la passe et on regarde la suivante.
          if(!line.empty())
            {

// !line.empty() donc on a rencontr une ligne non vide; on regarde alors si son format est valide.

            nbReadWords=0;
            std::istringstream lineStream(line);
            bool remainWordsToRead=true;
//  On lit dans la ligne line jusqu' ce qu'il n'y ait plus de mots dedans ou jusqu' ce du'on ait lu 3 mots dans line.
            while((remainWordsToRead)&&(nbReadWords<3))
              {

//  Il est permis de mettre des espaces entre chaque mot lu. Si on rencontre un mot vide, on le passe et on regarde le suivant.
              remainWordsToRead=std::getline(lineStream,word,',');// Entre chaque mot non vide le sparateur est une virgule.
              if(!word.empty())
                {
//  !word.empty() donc on a rencontr un mot non vide; on regarde alors si son format est valide.

//  Si le mot n'est pas de type entier et compris entre 0 et 9 (inclus), alors le fichier n'a pas le bon format et on lance une
//  exception de type std::invalid_argument.
                if((!soma::utils::general::isOfType<int>(tmpVal=-1,word))||(tmpVal<0)||(tmpVal>9))
                  {

                  fileStream.close();
                  throw
                      (
                      std::invalid_argument
                                          (
                                          "File format not valid!"
                                          )
                      );

                  }

//  On en profite pour rcuprer les coordonnes minimales et maximales lues pour chaque axe (0x), (0y) et (0z).
                if((cubeletsPositions[nbReadLines][nbReadWords]=(unsigned char)tmpVal)<minAndMax[nbReadWords][0])
                  minAndMax[nbReadWords][0]=cubeletsPositions[nbReadLines][nbReadWords];
                else
                  if(cubeletsPositions[nbReadLines][nbReadWords]>minAndMax[nbReadWords][1])
                    minAndMax[nbReadWords][1]=cubeletsPositions[nbReadLines][nbReadWords];
                ++nbReadWords;

                }

              }
//  Si on arrive  la fin du mot et qu'on n'a pas lu 3 entiers compris entre 0 et 9 (inclus) spars par des virgules
//  ou si le mot n'est pas fini, alors le fichier n'a pas le bon format et on lance une exception de type std::invalid_argument.
            if((nbReadWords<3)||(!lineStream.eof()))
              {

              fileStream.close();
              throw
                  (
                  std::invalid_argument
                                      (
                                      "File format not valid!"
                                      )
                  );

              }

            ++nbReadLines;

            }

          }

//  Si on arrive  la fin du fichier  fileName sans avoir lu 27 coordonnes au format valide ou si le fichier n'est pas
//  fini, alors celui-ci n'a pas le bon format et on lance une exception de type std::invalid_argument.
        if((nbReadLines<27)||(!fileStream.eof()))
          {

          fileStream.close();
          throw
              (
              std::invalid_argument
                                  (
                                  "File format not valid!"
                                  )
              );

          }

//  Arriv  ce stade la lecture du fichier fileName s'est bien droule et est finie. On peut donc librer le flux
//  de lecture fileStream.
        fileStream.close();

//  La donne membre m_BoundingBox est initialise avec les dimensions maximales sur chaque axe (0X), (0Y) et (0Z)
//  de la structure charge. Sur l'axe des X, m_BoundingBox compte ainsi minAndMax[0][1] (== X maximal) -
//  minAndMax[0][0] (== X minimal) +1 cubes. Sur l'axe des Y, m_BoundingBox a minAndMax[1][1] (== Y maximal) -
//  minAndMax[1][0] (== Y minimal) +1 cubes. Enfin, sur l'axe des Z, m_BoundingBox contient minAndMax[2][1] (== Z maximal)
//  - minAndMax[2][0] (== Z minimal) +1 cubes.
//  Au dpart, tous les lments de m_BoundingBox sont initialises  0; c'est--dire que la structure que l'on a charge
//  n'a pas ecore t dispose dans sa bote englobante
        m_BoundingBox=soma::utils::general::CBitArray3D
                                                      (
                                                      minAndMax[0][1]-minAndMax[0][0]+1,
                                                      minAndMax[1][1]-minAndMax[1][0]+1,
                                                      minAndMax[2][1]-minAndMax[2][0]+1,
                                                      false
                                                      );

//  La structure (et donc sa bote englobante) est centre en x==0.
        if(minAndMax[0][0]>0)
          {
          const unsigned char xMin=minAndMax[0][0];
          cubeletsPositions[0][0]-=xMin;
          cubeletsPositions[1][0]-=xMin;
          cubeletsPositions[2][0]-=xMin;
          cubeletsPositions[3][0]-=xMin;
          cubeletsPositions[4][0]-=xMin;
          cubeletsPositions[5][0]-=xMin;
          cubeletsPositions[6][0]-=xMin;
          cubeletsPositions[7][0]-=xMin;
          cubeletsPositions[8][0]-=xMin;
          cubeletsPositions[9][0]-=xMin;
          cubeletsPositions[10][0]-=xMin;
          cubeletsPositions[11][0]-=xMin;
          cubeletsPositions[12][0]-=xMin;
          cubeletsPositions[13][0]-=xMin;
          cubeletsPositions[14][0]-=xMin;
          cubeletsPositions[15][0]-=xMin;
          cubeletsPositions[16][0]-=xMin;
          cubeletsPositions[17][0]-=xMin;
          cubeletsPositions[18][0]-=xMin;
          cubeletsPositions[19][0]-=xMin;
          cubeletsPositions[20][0]-=xMin;
          cubeletsPositions[21][0]-=xMin;
          cubeletsPositions[22][0]-=xMin;
          cubeletsPositions[23][0]-=xMin;
          cubeletsPositions[24][0]-=xMin;
          cubeletsPositions[25][0]-=xMin;
          cubeletsPositions[26][0]-=xMin;
          }

//  La structure (et donc sa bote englobante)  est centre en y==0.
        if(minAndMax[1][0]>0)
          {
          const unsigned char yMin=minAndMax[1][0];
          cubeletsPositions[0][1]-=yMin;
          cubeletsPositions[1][1]-=yMin;
          cubeletsPositions[2][1]-=yMin;
          cubeletsPositions[3][1]-=yMin;
          cubeletsPositions[4][1]-=yMin;
          cubeletsPositions[5][1]-=yMin;
          cubeletsPositions[6][1]-=yMin;
          cubeletsPositions[7][1]-=yMin;
          cubeletsPositions[8][1]-=yMin;
          cubeletsPositions[9][1]-=yMin;
          cubeletsPositions[10][1]-=yMin;
          cubeletsPositions[11][1]-=yMin;
          cubeletsPositions[12][1]-=yMin;
          cubeletsPositions[13][1]-=yMin;
          cubeletsPositions[14][1]-=yMin;
          cubeletsPositions[15][1]-=yMin;
          cubeletsPositions[16][1]-=yMin;
          cubeletsPositions[17][1]-=yMin;
          cubeletsPositions[18][1]-=yMin;
          cubeletsPositions[19][1]-=yMin;
          cubeletsPositions[20][1]-=yMin;
          cubeletsPositions[21][1]-=yMin;
          cubeletsPositions[22][1]-=yMin;
          cubeletsPositions[23][1]-=yMin;
          cubeletsPositions[24][1]-=yMin;
          cubeletsPositions[25][1]-=yMin;
          cubeletsPositions[26][1]-=yMin;
          }

//  La structure (et donc sa bote englobante)  est centre en z==0.
        if(minAndMax[2][0]>0)
          {
          const unsigned char zMin=minAndMax[2][0];
          cubeletsPositions[0][2]-=zMin;
          cubeletsPositions[1][2]-=zMin;
          cubeletsPositions[2][2]-=zMin;
          cubeletsPositions[3][2]-=zMin;
          cubeletsPositions[4][2]-=zMin;
          cubeletsPositions[5][2]-=zMin;
          cubeletsPositions[6][2]-=zMin;
          cubeletsPositions[7][2]-=zMin;
          cubeletsPositions[8][2]-=zMin;
          cubeletsPositions[9][2]-=zMin;
          cubeletsPositions[10][2]-=zMin;
          cubeletsPositions[11][2]-=zMin;
          cubeletsPositions[12][2]-=zMin;
          cubeletsPositions[13][2]-=zMin;
          cubeletsPositions[14][2]-=zMin;
          cubeletsPositions[15][2]-=zMin;
          cubeletsPositions[16][2]-=zMin;
          cubeletsPositions[17][2]-=zMin;
          cubeletsPositions[18][2]-=zMin;
          cubeletsPositions[19][2]-=zMin;
          cubeletsPositions[20][2]-=zMin;
          cubeletsPositions[21][2]-=zMin;
          cubeletsPositions[22][2]-=zMin;
          cubeletsPositions[23][2]-=zMin;
          cubeletsPositions[24][2]-=zMin;
          cubeletsPositions[25][2]-=zMin;
          cubeletsPositions[26][2]-=zMin;
          }

//  Ainsi, quelque soit i allant de de 0  26, on a 0 <= cubeletsPositions[i][0] <= m_BoundingBox.size1() (le nombre maximal
//  de cubes sur l'axe des X de la structure). On a galement 0 <= cubeletsPositions[i][1] <= m_BoundingBox.size2()
//  (le nombre maximal de cubes sur l'axe des Y  de la structure). Et pour finir, on a :
//  0 <= ccubeletsPositions[i][2] <= m_BoundingBox.size3() (le nombre maximal de cubes sur l'axe des Z de la structure).
//  On peut donc crire l'instruction m_BoundingBox(cubeletsPositions[i][0],cubeletsPositions[i][1],cubeletsPositions[i][2])=true;
//  Cela revient  disposer les 27 cubes de la structure que l'on a charges dans sa bote englobante.
        m_BoundingBox(cubeletsPositions[0][0],cubeletsPositions[0][1],cubeletsPositions[0][2])=true;
        m_BoundingBox(cubeletsPositions[1][0],cubeletsPositions[1][1],cubeletsPositions[1][2])=true;
        m_BoundingBox(cubeletsPositions[2][0],cubeletsPositions[2][1],cubeletsPositions[2][2])=true;
        m_BoundingBox(cubeletsPositions[3][0],cubeletsPositions[3][1],cubeletsPositions[3][2])=true;
        m_BoundingBox(cubeletsPositions[4][0],cubeletsPositions[4][1],cubeletsPositions[4][2])=true;
        m_BoundingBox(cubeletsPositions[5][0],cubeletsPositions[5][1],cubeletsPositions[5][2])=true;
        m_BoundingBox(cubeletsPositions[6][0],cubeletsPositions[6][1],cubeletsPositions[6][2])=true;
        m_BoundingBox(cubeletsPositions[7][0],cubeletsPositions[7][1],cubeletsPositions[7][2])=true;
        m_BoundingBox(cubeletsPositions[8][0],cubeletsPositions[8][1],cubeletsPositions[8][2])=true;
        m_BoundingBox(cubeletsPositions[9][0],cubeletsPositions[9][1],cubeletsPositions[9][2])=true;
        m_BoundingBox(cubeletsPositions[10][0],cubeletsPositions[10][1],cubeletsPositions[10][2])=true;
        m_BoundingBox(cubeletsPositions[11][0],cubeletsPositions[11][1],cubeletsPositions[11][2])=true;
        m_BoundingBox(cubeletsPositions[12][0],cubeletsPositions[12][1],cubeletsPositions[12][2])=true;
        m_BoundingBox(cubeletsPositions[13][0],cubeletsPositions[13][1],cubeletsPositions[13][2])=true;
        m_BoundingBox(cubeletsPositions[14][0],cubeletsPositions[14][1],cubeletsPositions[14][2])=true;
        m_BoundingBox(cubeletsPositions[15][0],cubeletsPositions[15][1],cubeletsPositions[15][2])=true;
        m_BoundingBox(cubeletsPositions[16][0],cubeletsPositions[16][1],cubeletsPositions[16][2])=true;
        m_BoundingBox(cubeletsPositions[17][0],cubeletsPositions[17][1],cubeletsPositions[17][2])=true;
        m_BoundingBox(cubeletsPositions[18][0],cubeletsPositions[18][1],cubeletsPositions[18][2])=true;
        m_BoundingBox(cubeletsPositions[19][0],cubeletsPositions[19][1],cubeletsPositions[19][2])=true;
        m_BoundingBox(cubeletsPositions[20][0],cubeletsPositions[20][1],cubeletsPositions[20][2])=true;
        m_BoundingBox(cubeletsPositions[21][0],cubeletsPositions[21][1],cubeletsPositions[21][2])=true;
        m_BoundingBox(cubeletsPositions[22][0],cubeletsPositions[22][1],cubeletsPositions[22][2])=true;
        m_BoundingBox(cubeletsPositions[23][0],cubeletsPositions[23][1],cubeletsPositions[23][2])=true;
        m_BoundingBox(cubeletsPositions[24][0],cubeletsPositions[24][1],cubeletsPositions[24][2])=true;
        m_BoundingBox(cubeletsPositions[25][0],cubeletsPositions[25][1],cubeletsPositions[25][2])=true;
        m_BoundingBox(cubeletsPositions[26][0],cubeletsPositions[26][1],cubeletsPositions[26][2])=true;

        }



      }



    }



  }
