#ifndef BITARRAY3D_HPP
#define BITARRAY3D_HPP



#include <algorithm>
#include <stdexcept>
#include <vector>



namespace soma
  {



  namespace utils
    {



    namespace general
      {



      class CBitArray3D
        {

        public :

          typedef std::vector<bool>::value_type value_type;
          typedef const value_type const_value_type;

          typedef std::vector<bool>::difference_type difference_type;
          typedef const difference_type const_difference_type;

          typedef std::vector<bool>::size_type size_type;
          typedef const size_type const_size_type;

          typedef std::vector<bool>::reference reference;
          typedef std::vector<bool>::const_reference const_reference;

          typedef std::vector<bool>::pointer pointer;
          typedef std::vector<bool>::const_pointer const_pointer;

          typedef std::vector<bool>::iterator iterator;
          typedef std::vector<bool>::const_iterator const_iterator;

          typedef std::vector<bool>::const_reverse_iterator const_reverse_iterator;
          typedef std::vector<bool>::reverse_iterator reverse_iterator;



        private :

          size_type m_Size1,
                    m_Size2,
                    m_Size3;

          std::vector<bool> m_Data;


        public :

          CBitArray3D() : m_Size1(0),
                          m_Size2(0),
                          m_Size3(0),
                          m_Data() {}

          CBitArray3D
                    (
                    const_size_type N1,
                    const_size_type N2,
                    const_size_type N3
                    );



          CBitArray3D
                    (
                    const_size_type N1,
                    const_size_type N2,
                    const_size_type N3,
                    const_value_type fillVal
                    );



          inline CBitArray3D& resize
                                   (
                                   const_size_type newSize1,
                                   const_size_type newSize2,
                                   const_size_type newSize3,
                                   const bool fillVal=false
                                   );



          inline static const CBitArray3D generate3DChessBoard
                                                             (
                                                             const_size_type N1,
                                                             const_size_type N2,
                                                             const_size_type N3,
                                                             const_value_type firstVal=true
                                                             );



          inline iterator begin()
            {
            return
                 (
                 m_Data.begin()
                 );
            }

          inline const_iterator begin() const
            {
            return
                 (
                 m_Data.begin()
                 );
            }



          inline iterator end()
            {
            return
                 (
                 m_Data.end()
                 );
            }

          inline const_iterator end() const
            {
            return
                 (
                 m_Data.end()
                 );
            }



          inline reverse_iterator rbegin()
            {
            return
                 (
                 m_Data.rbegin()
                 );
            }

          inline const_reverse_iterator rbegin() const
            {
            return
                 (
                 m_Data.rbegin()
                 );
            }



          inline reverse_iterator rend()
            {
            return
                 (
                 m_Data.rend()
                 );
            }

          inline const_reverse_iterator rend() const
            {
            return
                 (
                 m_Data.rend()
                 );
            }



          inline const_size_type dim1() const
            {
            return
                 (
                 m_Size1
                 );
            }

          inline const_size_type size1() const
            {
            return
                 (
                 dim1()
                 );
            }



          inline const_size_type dim2() const
            {
            return
                 (
                 m_Size2
                 );
            }

          inline const_size_type size2() const
            {
            return
                 (
                 dim2()
                 );
            }



          inline const_size_type dim3() const
            {
            return
                 (
                 m_Size3
                 );
            }

          inline const_size_type size3() const
            {
            return
                 (
                 dim3()
                 );
            }



          inline const_size_type dim() const
            {
            return
                 (
                 m_Data.size()
                 );
            }

          inline const_size_type size() const
            {
            return
                 (
                 dim()
                 );
            }



          inline reference operator()
                                    (
                                    const_size_type i,
                                    const_size_type j,
                                    const_size_type k
                                    );

          inline const_reference operator()
                                          (
                                          const_size_type i,
                                          const_size_type j,
                                          const_size_type k
                                          ) const;



          inline reference at
                            (
                            const_size_type i,
                            const_size_type j,
                            const_size_type k
                            );

          inline const_reference at
                                  (
                                  const_size_type i,
                                  const_size_type j,
                                  const_size_type k
                                  ) const;



          inline CBitArray3D& flip();



          inline const CBitArray3D operator~() const;



          inline CBitArray3D& swap
                                 (
                                 CBitArray3D& arr
                                 );

          inline static void swap
                                (
                                reference x,
                                reference y
                                );

        };



      }



    }



  }



#endif  //  BITARRAY3D_HPP
